/*
flip51   Software for the ELV Flash Programmer FP51
Copyright (C) 1999-2003 Oliver Tscherwitschke <oliver@tscherwitschke.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef TARGET_LINUX
    #include <unistd.h>
    #include <asm/io.h>
#endif
#ifdef TARGET_WINDOWS
    #include <windows.h>
#endif

#include <stdio.h>

#include "types.h"
#include "delay.h"


#ifdef TARGET_WINDOWS
/****************************************/
/* small test for the delay function    */
/* works only on windows                */
/****************************************/
void test_speed(void)
{
    LARGE_INTEGER perf_freq;
    LARGE_INTEGER perf_count1;
    LARGE_INTEGER perf_count2;

    __int64 count1;
    __int64 count2;
    __int64 diff;
    __int64 offset;
    
          
    QueryPerformanceFrequency(&perf_freq);
    count1 =  perf_freq.QuadPart;
    printf("PerFreq: %d Hz\n", count1);   

    QueryPerformanceCounter(&perf_count1);
    QueryPerformanceCounter(&perf_count2);
    count1 = perf_count1.QuadPart;
    count2 = perf_count2.QuadPart;
    offset = count2 - count1;
  
    QueryPerformanceCounter(&perf_count1);
    uwait(100);
    QueryPerformanceCounter(&perf_count2);
    count1 = perf_count1.QuadPart;
    count2 = perf_count2.QuadPart;
    diff = count2 - count1 - offset;
    printf("Diff: %i us\n", diff * 1000000 / (__int64) perf_freq.QuadPart);   
}
#endif


#ifdef TARGET_WINDOWS
/********************************************************/
/* uwait()     wait specified number of micro seconds   */
/* works only on windows                                */
/********************************************************/
void uwait(unsigned long us)
{
    LARGE_INTEGER perf_count1;
    LARGE_INTEGER perf_count2;
    static LARGE_INTEGER perf_freq;
    static int first_run = 1;  
    LONGLONG perf;

    if (first_run)
    {
        if (QueryPerformanceFrequency(&perf_freq) == 0)
        {
            printf("error: hardware doesn't support performance counter.\n");
            exit(1);
        }        
        first_run = 0;
    }
         
    QueryPerformanceCounter(&perf_count1);

    perf = (us * perf_freq.QuadPart / 1000000);
    do
    {
        QueryPerformanceCounter(&perf_count2);
    }
    while ((perf_count2.QuadPart - perf_count1.QuadPart) < perf);        
}
#endif
