/*
flip51   Software for the ELV Flash Programmer FP51
Copyright (C) 1999-2003 Oliver Tscherwitschke <oliver@tscherwitschke.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _FLIP51_H_
#define _FLIP51_H_

#include <stdio.h>

#include "types.h"


#ifdef TARGET_LINUX
	#define USLEEP(us)	        usleep(us)
    #define OUTB(val, port)     outb(val, port)
    #define INB(port)           inb(port)
#endif
#ifdef TARGET_WINDOWS
	#define USLEEP(us)          uwait(us)
    #define OUTB(val, port)     _outp((unsigned short) (port), (int) (val))
    #define INB(port)           _inp((unsigned short) (port))
#endif



/* defines */
#define FLIPVERSION "0.3.1"

/* variables */
extern unsigned short  baseport;           /* i/o adress of parallel port */



/* function declarations for flip51 */
void fp51_reset();
int  fp51_check();
void chip_erase(CHIP_INFO *chipinfo);
int  arguments(int argc, char* argv[]);
byte read_byte();
int read_sig(CHIP_INFO *chipinfo);
void prog_lockbits();
void read_flash(byte *buffer, CHIP_INFO *chipinfo, int readsize);
int  write_flash(byte *buffer, CHIP_INFO *chipinfo, int size);
long read_bin_file(byte *buffer, unsigned long flashsize, FILE *infile);
long read_hex_file(byte *buffer, unsigned long flashsize, FILE *infile);

void flip51_exit(int);
int PortDriverInstall(unsigned char *path);
int PortDriverRemove(void);
int PortDriverStart(void);
int PortDriverStop(void);

#endif
