/*
flip51   Software for the ELV Flash Programmer FP51
Copyright (C) 1999-2003 Oliver Tscherwitschke <oliver@tscherwitschke.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>

#include "types.h"
#include "ihex.h"
#include "tools.h"

/****************************************************************/
/* ihex_checksum(): checks the Checksum of one I-Hex record     */
/* Input: rec:   pointer to record string                       */
/* Return:       0:     Checksum OK                             */
/*               1:     Bad checksum                            */
/****************************************************************/
byte ihex_checksum(byte rec[])
{
	unsigned char i;
	unsigned char reclen;
	unsigned char bufflen;
	unsigned char checksum = 0;
	
	bufflen = strlen(rec);

	if (bufflen < 11)			/* string too short */
		return 1;
	
	if (rec[0] != ':')			/* no i-hex line */
		return 1;
	
	reclen = _atoc(rec + 1);
	
	if (11 + reclen * 2 != bufflen)
		return 1;				/* wrong length */

	for (i = 0; i < 5 + reclen; i++)
	{
		checksum += _atoc(rec + 1 + 2 * i);
	}
	
	if (checksum != 0)			/* bad checksum */
		return 1;
		
	return 0;					/* everything OK */
}


/****************************************************************/
/* ihex_load_record(): load one i-hex record to buffer          */
/* Input:        sBuff: string                                  */
/* Return:       0:              OK                             */
/*               1:             End of Intel-Hex record         */
/*               2:             unsupported hex record          */
/*               3:             address out of range            */
/****************************************************************/
unsigned char ihex_load_record(byte rec[], byte *buffer, unsigned long flashsize, unsigned long *maxaddr)
{
	unsigned short i;
	
	unsigned short ucRecLen;
	unsigned short usiOffset;
	unsigned char ucRecType;
	unsigned char ucData;
	
	ucRecLen = _atoc(rec + 1);
	usiOffset = _atos(rec + 3);
	ucRecType = _atoc(rec + 7);
	
	if (ucRecType == 1)					/* end record */
		return 1;

	if (ucRecType != 0)					/* no data record */
		return 2;
		
	for (i = 0; i < ucRecLen; i++)
	{
		ucData = _atoc(rec + 9 + i * 2);
		/* program byte 'ucData' to Address 'usiOffset + i' */
        if ((unsigned long) (usiOffset + i) < flashsize)
            buffer[usiOffset + i] = ucData;
        else
            return 3;       /* address out of range */		
	}
	
	*maxaddr = usiOffset + ucRecLen - 1;		/* return max addr */
	
	return 0;
}

