/*
flip51   Software for the ELV Flash Programmer FP51
Copyright (C) 1999-2003 Oliver Tscherwitschke <oliver@tscherwitschke.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "types.h"
#include "flip51.h"
#include "delay.h"

#ifdef TARGET_WINDOWS
    #include <conio.h>
#endif

byte    latch7 = 0;                         /* value stored in latches */
byte    latch13 = 0;
byte    latch14 = 0;


/****************************************************************/
/* write a byte to the latch ic7                                */
/****************************************************************/
void latch_7(int value)
{
    latch7 = value;                                 /* remember value */
    OUTB(value, baseport);                          /* outbut data */
    USLEEP(IOWAIT);
    OUTB(INB(baseport + 2) | 1, baseport + 2);      /* set nStrobe low */
    USLEEP(IOWAIT);
    OUTB(INB(baseport + 2) & ~1, baseport + 2);   /* set nStrobe high */
}

/****************************************************************/
/* write a byte to one of the latches ic12...ic14               */
/* Addresses: 1, 2 and 3                                        */
/****************************************************************/
void write_byte_to_latch(int value, byte adr)
{
    /* have to remmber the value? */
    if (adr == 2)
        latch13 = value;
    else if (adr == 3)
        latch14 = value;

    latch_7((latch7 & 0xf8) | adr);                 /* put Address in latch     */
    OUTB(value, baseport);                          /* put value on data-bus    */
    USLEEP(IOWAIT);
    OUTB(INB(baseport + 2) | 2, baseport + 2);      /* enable address-decoder (clk low) */
    USLEEP(IOWAIT);
    OUTB(INB(baseport + 2) & ~2, baseport + 2);     /* disable address-decoder (clk hi) */
    USLEEP(IOWAIT);
}



/****************************************************************/
/* Wait until RDY line goes high                                */
/****************************************************************/
void wait_for_ready()
{
    latch_7(latch7 | 64);           /* enable rdy read */
    while(!(INB(baseport + 1) & 64));
    latch_7(latch7 & ~64);          /* disable rdy read */
}

/****************************************************************/
/* set programming mode                     */
/****************************************************************/
void set_mode(byte mode)
{
    write_byte_to_latch((latch14 & 0xe1) | (mode << 1), 3);
}

